@@ -14,15 +14,16 @@ except ImportError:
     # Fallback for environments without Pydantic
     BaseModel = object
 
-    def Field(*args, **kwargs):
-        return None
+    # Fallback Field function with same signature as Pydantic's Field
+    def Field(default=None, *args, **kwargs):  # noqa: ARG001
+        return default
 
     ConfigDict = dict
     PYDANTIC_AVAILABLE = False
-
+    # Remove the fallback Field function and ConfigDict since Pydantic is always available
 if TYPE_CHECKING:
     from .element_store import ElementStore
-    from .meta import MetaNode, MetaPropertyType
+    from .meta import MetaNode, MetaPropertyType, ValidationResult
 
 
 class NodeTypes(StrEnum):
@@ -77,13 +78,13 @@ if PYDANTIC_AVAILABLE:
         name: Optional[str] = Field(
             None, min_length=0, max_length=200
         )  # Allow empty strings
-        description: Optional[str] = Field(None, max_length=2000)
+        description: Optional[str] = Field(None, max_length=20000)
         tags: Optional[List[str]] = Field(None)
-        created_time: Optional[int] = Field(None, ge=0)
-        created_by: Optional[str] = Field(None, min_length=1, max_length=100)
-        updated_time: Optional[int] = Field(None, ge=0)
-        approved_time: Optional[int] = Field(None, ge=0)
-        approved_by: Optional[str] = Field(None, min_length=1, max_length=100)
+        created_time: Optional[Union[int, str]] = Field(None, ge=0)
+        created_by: Optional[Union[str, int]] = Field(None)
+        updated_time: Optional[Union[int, str]] = Field(None, ge=0)
+        approved_time: Optional[Union[int, str]] = Field(None, ge=0)
+        approved_by: Optional[Union[str, int]] = Field(None)
         start: Optional[Any] = Field(None)
         end: Optional[Any] = Field(None)
         original: Optional[Any] = Field(None)
@@ -116,17 +117,18 @@ if PYDANTIC_AVAILABLE:
                 extra = getattr(self, "__pydantic_extra__", {})
                 if key in extra:
                     return extra[key]
-                raise KeyError(key)
+                raise KeyError(key) from None
 
         def __setitem__(self, key: str, value: PropertyValue) -> None:
             """Set property value by key (dict-like access)."""
-            try:
-                setattr(self, key, value)
-            except ValueError:
-                # If validation fails for known field, store in extra
-                if hasattr(self.__class__, key):
-                    raise  # Re-raise validation error
-                # Set as extra property
+            # Check if this is a known model field
+            if key in self.__class__.model_fields:
+                try:
+                    setattr(self, key, value)
+                except ValueError:
+                    raise
+            else:
+                # For unknown fields, store in extra properties
                 if not hasattr(self, "__pydantic_extra__"):
                     object.__setattr__(self, "__pydantic_extra__", {})
                 self.__pydantic_extra__[key] = value
@@ -342,6 +344,40 @@ else:
             except KeyError:
                 return default
 
+        def __getattr__(self, name: str) -> Any:
+            """Get attribute from extra_properties if not found in __slots__."""
+            # Avoid infinite recursion with _KNOWN_ATTRS lookup
+            if name in ("_KNOWN_ATTRS", "extra_properties"):
+                raise AttributeError(
+                    f"'{type(self).__name__}' object has no attribute '{name}'"
+                )
+
+            # Try to get from extra_properties if not in __slots__
+            try:
+                extra = object.__getattribute__(self, "extra_properties")
+                if name in extra:
+                    return extra[name]
+            except AttributeError:
+                pass
+
+            raise AttributeError(
+                f"'{type(self).__name__}' object has no attribute '{name}'"
+            )
+
+        def __setattr__(self, name: str, value: Any) -> None:
+            """Set attribute, storing non-slot attributes in extra_properties."""
+            # Handle __slots__ attributes directly
+            if name in self.__slots__:
+                object.__setattr__(self, name, value)
+            else:
+                # Store extra attributes in extra_properties
+                try:
+                    extra = object.__getattribute__(self, "extra_properties")
+                    extra[name] = value
+                except AttributeError:
+                    # During __init__ before extra_properties exists
+                    object.__setattr__(self, name, value)
+
 
 @dataclass(slots=True)
 class ElementDetails:
@@ -368,10 +404,64 @@ class KG(MutableMapping[str, "BaseElement"]):
 
     __slots__ = ("_elements",)
 
-    def __init__(self) -> None:
-        """Initialize KG container."""
+    def __init__(
+        self,
+        elements: Optional[Union[List["BaseElement"], Dict[str, "BaseElement"]]] = None,
+    ) -> None:
+        """Initialize KG container.
+
+        Args:
+            elements: Optional list of elements or dictionary of id->element to initialize with.
+        """
         self._elements: Dict[str, BaseElement] = {}
 
+        if elements:
+            if isinstance(elements, dict):
+                self._elements.update(elements)
+            elif isinstance(elements, list):
+                for element in elements:
+                    if hasattr(element, "id"):
+                        self._elements[element.id] = element
+                    else:
+                        # Handle dicts if passed (though type hint says BaseElement)
+                        if isinstance(element, dict) and "id" in element:
+                            # We can't easily convert dict to BaseElement here without circular imports
+                            # so we just store it if it's a dict, but this breaks type safety.
+                            # Ideally elements should be BaseElement instances.
+                            self._elements[element["id"]] = element  # type: ignore
+
+    def add_node(
+        self, node_or_class_id: Union[str, "BaseElement"], **kwargs
+    ) -> "BaseElement":
+        """Add a node to the KG.
+
+        Args:
+            node_or_class_id: Either a class_id string to create a new Node, or an existing Node/BaseElement instance.
+            **kwargs: Properties for the node when creating a new one.
+
+        Returns:
+            The added element.
+        """
+        print("add_node called with:", node_or_class_id, kwargs)
+        if isinstance(node_or_class_id, str):
+            # Create new Node
+            try:
+                from .node import Node
+
+                element = Node(node_or_class_id, **kwargs)
+            except ImportError:
+                # Fallback if Node cannot be imported (should not happen in normal usage)
+                raise ImportError("Could not import Node class to create new node")
+        else:
+            # Assume it's an element
+            element = node_or_class_id
+
+        self._elements[element.id] = element
+        return element
+
+    # Alias for backward compatibility if needed, but prefer add_node
+    addNode = add_node
+
     def __getitem__(self, key: str) -> "BaseElement":
         """Get element by ID."""
         return self._elements[key]
@@ -450,7 +540,9 @@ class BaseElement(BaseElementEventEmitter):
         "_initialized",
     )
 
-    def __init__(self, data: ElementDetails, store: "ElementStore") -> None:
+    def __init__(
+        self, data: ElementDetails, store: Optional["ElementStore"] = None
+    ) -> None:
         """Initialize BaseElement with type safety and validation."""
         super().__init__()
 
@@ -463,7 +555,7 @@ class BaseElement(BaseElementEventEmitter):
         self.id: str = data.id
         self.class_id: str = data.class_id
         self.type: NodeTypes = data.type
-        self.store: ElementStore = store
+        self.store: Optional[ElementStore] = store
 
         # Optional properties with proper defaults
         self.to_id: Optional[str] = data.to_id
@@ -485,6 +577,10 @@ class BaseElement(BaseElementEventEmitter):
         self._initialized: bool = False
         self._initialize_element()
 
+    def set_store(self, store: "ElementStore") -> None:
+        """Set the element store reference."""
+        self.store = store
+
     async def complete_data(self) -> None:
         """Complete data initialization."""
         pass
@@ -577,13 +673,37 @@ class BaseElement(BaseElementEventEmitter):
     @property
     def meta_node(self) -> Optional["MetaNode"]:
         """Get the MetaNode that describes this BaseElement."""
-        return self.store.get_meta_node_by_id(self.class_id)
+        return self.store.get_meta_node_by_id(self.class_id) if self.store else None
 
     def set_default_properties(self) -> None:
         """Set default properties based on meta node."""
         if self.meta_node and hasattr(self.meta_node, "set_default_properties"):
             self.meta_node.set_default_properties(self)
 
+    def validate_against_schema(self) -> "ValidationResult":
+        """Validate element's properties against MetaNode's JSON schema.
+
+        Returns:
+            ValidationResult with valid=True/False and optional error list.
+            Returns valid=True if meta_node is None (implicit pass when schema unavailable).
+        """
+        if not self.meta_node:
+            # Import here to avoid circular dependency at module level
+            from .meta import ValidationResult
+
+            return ValidationResult(valid=True, errors=None)
+        return self.meta_node.validate(self.properties)
+
+    @property
+    def is_valid_against_schema(self) -> bool:
+        """Quick boolean check if element is valid against schema.
+
+        Returns:
+            True if validation passes or schema unavailable.
+            False if validation fails.
+        """
+        return self.validate_against_schema().valid
+
     def to_dict(self) -> Dict[str, Any]:
         """Convert element to dictionary format."""
         return {
@@ -618,12 +738,12 @@ class BaseElement(BaseElementEventEmitter):
             if isinstance(prop_value, list):
                 elements = []
                 for element_id in prop_value:
-                    element = self.store.get_element(element_id)
+                    element = self.store.get_element(element_id) if self.store else None
                     if element:
                         elements.append(element)
                 return elements
             else:
-                element = self.store.get_element(prop_value)
+                element = self.store.get_element(prop_value) if self.store else None
                 return [element] if element else []
         return []
 
@@ -631,7 +751,7 @@ class BaseElement(BaseElementEventEmitter):
         """Get single element referenced by a property."""
         if prop in self.properties:
             element_id = self.properties[prop]
-            return self.store.get_element(element_id)
+            return self.store.get_element(element_id) if self.store else None
         return None
 
     def get_hash(self) -> int:
