@@ -19,8 +19,6 @@ from .node import Node
 if TYPE_CHECKING:
     from .graph_query import GraphQuery
 
-
-class ClassType:
     """Type hint for element classes."""
 
     pass
@@ -68,17 +66,87 @@ class IDataStorage(ABC):
         """Update elements in storage."""
         pass
 
+    @abstractmethod
+    async def patch_elements(self, elements: List[BaseElement]) -> None:
+        """Partially update elements in storage."""
+        pass
+
     @abstractmethod
     async def delete_elements(self, elements: List[BaseElement]) -> None:
         """Delete elements from storage."""
         pass
 
+    async def get_elements_by_class_id(self, class_id: str) -> Dict[str, BaseElement]:
+        """
+        Get elements by class ID from storage.
+
+        Args:
+            class_id: The class ID to filter by
+
+        Returns:
+            Dictionary of BaseElement objects keyed by ID that match the class
+        """
+        # Default implementation for storage backends that don't support native filtering
+        data = await self.load_data()
+        if not data:
+            return {}
+
+        return {
+            element_id: element
+            for element_id, element in data.items()
+            if element.class_id == class_id
+        }
+
     @abstractmethod
     async def reset(self) -> None:
         """Reset/clear all data in storage."""
         pass
 
 
+def create_data_storage(config: IDataStorageConfig) -> IDataStorage:
+    """
+    Factory function to create the appropriate data storage based on config.
+
+    Args:
+        config: Storage configuration
+
+    Returns:
+        Configured data storage instance
+    """
+    storage_type = config.type.lower()
+
+    if storage_type == "memory":
+        return MemoryDataStorage()
+    elif storage_type == "rest":
+        # Import here to avoid circular imports
+        from .rest_client import RestDataStorage, RestDataStorageConfig
+
+        if isinstance(config, RestDataStorageConfig):
+            return RestDataStorage(config.to_rest_config())
+        else:
+            # Create default REST config from extra_config
+            rest_config = RestDataStorageConfig(**config.extra_config)
+            return RestDataStorage(rest_config.to_rest_config())
+    elif storage_type == "synchronized":
+        # Import here to avoid circular imports
+        from .rest_client import SynchronizedDataStorage, SynchronizedDataStorageConfig
+
+        if isinstance(config, SynchronizedDataStorageConfig):
+            storage = SynchronizedDataStorage(config.to_rest_config())
+            if config.json_store_path:
+                storage.set_json_store_path(config.json_store_path)
+            return storage
+        else:
+            # Create default synchronized config from extra_config
+            sync_config = SynchronizedDataStorageConfig(**config.extra_config)
+            storage = SynchronizedDataStorage(sync_config.to_rest_config())
+            if sync_config.json_store_path:
+                storage.set_json_store_path(sync_config.json_store_path)
+            return storage
+    else:
+        raise ValueError(f"Unknown storage type: {storage_type}")
+
+
 class MemoryDataStorage(IDataStorage):
     """Simple in-memory data storage implementation."""
 
@@ -111,6 +179,12 @@ class MemoryDataStorage(IDataStorage):
             if element.id in self._data:
                 del self._data[element.id]
 
+    async def patch_elements(self, elements: List[BaseElement]) -> None:
+        """Patch elements in memory."""
+        for element in elements:
+            if element.id in self._data:
+                self._data[element.id] = element
+
     async def reset(self) -> None:
         """Clear all data from memory."""
         self._data.clear()
@@ -145,7 +219,13 @@ class ElementStore:
         self.elements = KG()
 
         if config:
-            self.data_operation = config.operation or MemoryDataStorage()
+            if config.operation:
+                self.data_operation = config.operation
+            elif config.data_config:
+                self.data_operation = create_data_storage(config.data_config)
+            else:
+                self.data_operation = MemoryDataStorage()
+
             if config.data_config:
                 self.data_operation.store_server = config.data_config
             self.default_node = config.default_node or Node
@@ -238,7 +318,7 @@ class ElementStore:
 
     def query(
         self,
-        elements: Optional[List[BaseElement]] = None,
+        elements: Optional[List[BaseElement]] = None,  # noqa: ARG002
         options: Optional[QueryOptions] = None,
     ) -> "GraphQuery":
         """Create a new graph query."""
@@ -250,7 +330,7 @@ class ElementStore:
         return query
 
     def create_instance(
-        self, class_id: str, element_type: str, *args, **kwargs
+        self, class_id: str, element_type: str, *args, **kwargs  # noqa: ARG002
     ) -> BaseElement:
         """Create instance of registered element type."""
         if class_id in self.element_types:
@@ -312,7 +392,6 @@ class ElementStore:
                 and element_data.class_id in self.elements
                 and self.elements[element_data.class_id].type == NodeTypes.META
             ):
-
                 metanode = self.elements[element_data.class_id]
 
                 if not element_data.attributes:
@@ -387,7 +466,7 @@ class ElementStore:
             ),
             source=(
                 getattr(preset, "source", None)
-                or (meta_node.properties.target_source if meta_node else None)
+                or (meta_node.properties.get("target_source") if meta_node else None)
                 or (meta_node.source if meta_node else None)
             ),
             temp=getattr(preset, "temp", False),
@@ -431,9 +510,9 @@ class ElementStore:
             BaseElement, ElementDetails, List[Union[BaseElement, ElementDetails]]
         ],
         skip_save: bool = False,
-        notify: bool = True,
+        notify: bool = True,  # noqa: ARG002
         lazy: bool = False,
-        datasource: Optional[str] = None,
+        datasource: Optional[str] = None,  # noqa: ARG002
     ) -> List[BaseElement]:
         """Add elements to store and save to data storage."""
         added_elements = []
