@@ -1,6 +1,6 @@
 import logging
 import re
-from dataclasses import dataclass
+from dataclasses import asdict, dataclass
 from enum import Enum
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union
 
@@ -256,6 +256,7 @@ class GraphQuery(FluentQuery):
                 method=QueryOperations.FILTER_BY_SOURCE,
                 params={"source": source},
                 args=[source, options.is_live if options else False],
+                options=options,
             )
         )
 
@@ -279,7 +280,9 @@ class GraphQuery(FluentQuery):
         return self
 
     def filter_by_bounding_box(
-        self, bounding_box: List[float], options: Optional[IFluentOptions] = None
+        self,
+        bounding_box: List[float],
+        options: Optional[IFluentOptions] = None,  # noqa: ARG002
     ) -> "GraphQuery":
         """Filter elements by spatial bounding box."""
         if bounding_box and len(bounding_box) == 4:
@@ -667,7 +670,7 @@ class GraphQuery(FluentQuery):
             queue = []
 
             # Seed initial neighbors
-            def add_to_queue(node_id):
+            def add_to_queue(node_id):  # noqa: B023
                 if node_id not in visited:
                     visited.add(node_id)
                     queue.append({"id": node_id, "depth": 1})
@@ -691,7 +694,7 @@ class GraphQuery(FluentQuery):
                 if not node:
                     continue
 
-                def add_next_level(node_id):
+                def add_next_level(node_id):  # noqa: B023
                     if node_id not in visited:
                         visited.add(node_id)
                         queue.append({"id": node_id, "depth": current["depth"] + 1})
@@ -1299,7 +1302,9 @@ class GraphQuery(FluentQuery):
         return self
 
     def filter_by_link_element(
-        self, link_element: BaseElement, options: Optional[IFluentOptions] = None
+        self,
+        link_element: BaseElement,
+        options: Optional[IFluentOptions] = None,  # noqa: ARG002
     ) -> "GraphQuery":
         """Filter by link element."""
         element_details = getattr(link_element.properties, "element_details", {})
@@ -1462,6 +1467,17 @@ class GraphQuery(FluentQuery):
 
         return self
 
+    # Backwards-compatible alias used by some helper methods
+    def filter_by_property(
+        self,
+        key: str,
+        operator: FilterOperator,
+        value: PropertyValue,
+        options: Optional[IFluentOptions] = None,
+    ) -> "GraphQuery":
+        """Alias for where(); kept for compatibility with older call sites."""
+        return self.where(key, operator, value, options)
+
     def _handle_relation_filter(
         self,
         element: BaseElement,
@@ -2437,7 +2453,31 @@ class GraphQuery(FluentQuery):
                 {
                     "method": op.method.value,
                     "params": op.params,
-                    "options": vars(op.options) if op.options else None,
+                    "options": asdict(op.options) if op.options else None,
                 }
             )
         return {"operations": operations}
+
+    @staticmethod
+    def from_json(data: Dict[str, Any], store: "ElementStore") -> "GraphQuery":
+        """Recreate a GraphQuery from a JSON payload produced by to_json().
+
+        This mirrors the TypeScript `deserializeQuery` that takes a list of
+        operations and replays them. Only the serializable subset of
+        IFluentOperation is supported (method/params/options).
+        """
+
+        operations: List[IFluentOperation] = []
+        for op in data.get("operations", []) or []:
+            method = QueryOperations(op["method"])
+            raw_opts = op.get("options") or None
+            opts = IFluentOptions(**raw_opts) if raw_opts else None
+            operations.append(
+                IFluentOperation(
+                    method=method,
+                    params=op.get("params") or None,
+                    options=opts,
+                )
+            )
+
+        return GraphQuery.deserialize_query(operations, store)
