@@ -1,4 +1,4 @@
-from dataclasses import dataclass, field
+from dataclasses import dataclass, field, fields
 from enum import StrEnum
 from functools import lru_cache
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union
@@ -106,6 +106,8 @@ class PropertyRelation:
     type: Optional[str] = None
     target: Optional[str] = None
     direction: Optional[str] = None  # 'in' | 'out' | 'both'
+    multiple: Optional[bool] = None
+    targetTypes: Optional[List[str]] = None
 
     def __post_init__(self):
         """Validate relation configuration."""
@@ -157,6 +159,7 @@ class MetaPropertyType:
 
     # Options and lists
     options: Optional[List[str]] = None
+    aliases: Optional[List[str]] = None
     suggestions_list: Optional[str] = None
     labels: Optional[List[str]] = None
     list: Optional[List[ListItem]] = None
@@ -338,7 +341,9 @@ class MetaNode(BaseElement):
     """Extended MetaNode class for metadata definitions."""
 
     def __init__(
-        self, data: Union[MetaNodeDetails, ElementDetails], store: "ElementStore"
+        self,
+        data: Union[MetaNodeDetails, ElementDetails],
+        store: Optional["ElementStore"] = None,
     ):
         """Initialize MetaNode."""
         super().__init__(data, store)
@@ -384,6 +389,55 @@ class MetaNode(BaseElement):
             return
 
         for key, pt in property_types.items():
+            # Convert dict to MetaPropertyType if needed
+            if isinstance(pt, dict):
+                # Handle nested conversions
+                if "relation" in pt and isinstance(pt["relation"], dict):
+                    # Handle migration from objectType to targetTypes
+                    if "objectType" in pt["relation"]:
+                        obj_type = pt["relation"].pop("objectType")
+                        if obj_type and "targetTypes" not in pt["relation"]:
+                            pt["relation"]["targetTypes"] = [obj_type]
+
+                    pt["relation"] = PropertyRelation(**pt["relation"])
+
+                if "monitor" in pt and isinstance(pt["monitor"], dict):
+                    pt["monitor"] = MonitorConfig(**pt["monitor"])
+
+                if "list" in pt and isinstance(pt["list"], list):
+                    pt["list"] = [
+                        ListItem(**item) if isinstance(item, dict) else item
+                        for item in pt["list"]
+                    ]
+
+                if "tasks" in pt and isinstance(pt["tasks"], list):
+                    pt["tasks"] = [
+                        AgentTask(**item) if isinstance(item, dict) else item
+                        for item in pt["tasks"]
+                    ]
+
+                if "conditions" in pt and isinstance(pt["conditions"], list):
+                    pt["conditions"] = [
+                        PropertyCondition(**item) if isinstance(item, dict) else item
+                        for item in pt["conditions"]
+                    ]
+
+                # Filter keys to match MetaPropertyType fields
+                valid_keys = {f.name for f in fields(MetaPropertyType)}
+                filtered_pt = {k: v for k, v in pt.items() if k in valid_keys}
+
+                # Ensure required fields are present or have defaults
+                if "key" not in filtered_pt:
+                    filtered_pt["key"] = key
+                if "label" not in filtered_pt:
+                    filtered_pt["label"] = format_title(key)
+                if "type" not in filtered_pt:
+                    # Default to string if type is missing
+                    filtered_pt["type"] = PropValueType.STRING
+
+                pt = MetaPropertyType(**filtered_pt)
+                property_types[key] = pt
+
             # Set defaults
             if not pt.key:
                 pt.key = key
@@ -497,6 +551,264 @@ class MetaNode(BaseElement):
 
         return defaults
 
+    # ------------------------------------------------------------------
+    # Filtering helpers
+    # ------------------------------------------------------------------
+
+    @property
+    def filterable_properties(self) -> List[MetaPropertyType]:
+        """Return property types that should be exposed as filters.
+
+        The tests treat properties with ``prio < 2`` as filterable.
+        """
+
+        return [pt for pt in self.all_prop_types.values() if pt.prio and pt.prio < 2]
+
+    # ------------------------------------------------------------------
+    # Inheritance helpers
+    # ------------------------------------------------------------------
+
+    @property
+    def inherited_meta_nodes(self) -> List["MetaNode"]:
+        """Get inherited meta nodes (resolved from ``inherits_from``)."""
+
+        if self._inherited_meta_nodes is None:
+            self._inherited_meta_nodes = self._calculate_inherited_meta_nodes()
+        return self._inherited_meta_nodes
+
+    @property
+    def inherited_meta_nodes_names(self) -> List[str]:
+        """Return the ids of all inherited meta nodes."""
+
+        return [meta_node.id for meta_node in self.inherited_meta_nodes]
+
+    def inherits_from(self, class_id: str) -> bool:
+        """Return True if this meta node inherits from ``class_id``.
+
+        This includes the node itself so ``inherits_from(self.class_id)``
+        is also considered ``True``.
+        """
+
+        return self.class_id == class_id or class_id in self.inherited_meta_nodes_names
+
+    def _calculate_inherited_meta_nodes(self) -> List["MetaNode"]:
+        """Resolve the inheritance chain to MetaNode instances."""
+
+        if not self.store:
+            return []
+
+        visited_nodes: set[str] = set()
+        inherited_meta_nodes: List[MetaNode] = []
+
+        def collect(node_ids: List[str]) -> None:
+            for parent_id in node_ids:
+                if parent_id in visited_nodes:
+                    continue
+
+                visited_nodes.add(parent_id)
+                parent = self.store.elements.get(parent_id)
+
+                if parent and isinstance(parent, MetaNode):
+                    inherited_meta_nodes.append(parent)
+                    inherits = getattr(parent.properties, "inherits_from", None)
+                    if inherits:
+                        collect(inherits)
+
+        inherits_from = getattr(self.properties, "inherits_from", None)
+        if inherits_from:
+            collect(inherits_from)
+
+        return inherited_meta_nodes
+
+    def _gather_inherited_attributes(self) -> Dict[str, Any]:
+        """Gather attributes from the inheritance chain.
+
+        Later nodes in the chain override earlier ones; ``card:sections``
+        are merged by ``type``.
+        """
+
+        inherited_attributes: Dict[str, Any] = {}
+        all_nodes: List[MetaNode] = list(self.inherited_meta_nodes) + [self]
+
+        for meta_node in all_nodes:
+            if not meta_node.attributes:
+                continue
+
+            for key, value in meta_node.attributes.items():
+                if key == "card:sections" and key in inherited_attributes and value:
+                    sections_by_type: Dict[str, Dict[str, Any]] = {}
+
+                    # existing
+                    for section in inherited_attributes[key]:
+                        if isinstance(section, dict) and "type" in section:
+                            sections_by_type[section["type"]] = section
+
+                    # new
+                    for section in value:
+                        if isinstance(section, dict) and "type" in section:
+                            stype = section["type"]
+                            if stype in sections_by_type:
+                                merged = {**sections_by_type[stype], **section}
+                                if "properties" in section:
+                                    existing_props = sections_by_type[stype].get(
+                                        "properties", []
+                                    )
+                                    merged["properties"] = list(
+                                        set(existing_props + section["properties"])
+                                    )
+                                sections_by_type[stype] = merged
+                            else:
+                                sections_by_type[stype] = section
+
+                    inherited_attributes[key] = list(sections_by_type.values())
+                else:
+                    if (
+                        key not in inherited_attributes
+                        or inherited_attributes[key] != value
+                    ):
+                        inherited_attributes[key] = value
+
+        return inherited_attributes
+
+    def _gather_inherited_property_types(self) -> Dict[str, MetaPropertyType]:
+        """Gather property types from this node and its ancestors."""
+
+        all_nodes: List[MetaNode] = list(self.inherited_meta_nodes) + [self]
+        result: Dict[str, MetaPropertyType] = {}
+
+        for meta_node in all_nodes:
+            prop_types = getattr(meta_node.properties, "property_types", None)
+            if prop_types:
+                result.update(prop_types)
+
+        return result
+
+    def _gather_inherited_properties(self) -> MetaNodeProperties:
+        """Build a properties object with inherited scalar attributes."""
+
+        all_nodes: List[MetaNode] = list(self.inherited_meta_nodes) + [self]
+        inheritable = [
+            "icon",
+            "description",
+            "color",
+            "storage",
+            "readonly",
+            "ttl_update",
+            "ttl_creation",
+            "default_legend_property",
+        ]
+
+        inherited_values: Dict[str, Any] = {}
+
+        for meta_node in all_nodes:
+            for prop in inheritable:
+                value = getattr(meta_node.properties, prop, None)
+                if value is not None:
+                    inherited_values[prop] = value
+
+        result_props = MetaNodeProperties(**vars(self.properties))
+        for prop, value in inherited_values.items():
+            setattr(result_props, prop, value)
+
+        return result_props
+
+    # ------------------------------------------------------------------
+    # JSON Schema / validation helpers
+    # ------------------------------------------------------------------
+
+    def to_json_schema(
+        self, options: Optional[Dict[str, Any]] = None
+    ) -> Dict[str, Any]:
+        """Convert this MetaNode to a JSON Schema dict.
+
+        Delegates to the module-level ``meta_node_to_json_schema`` helper so
+        behaviour stays aligned with other callers.
+        """
+
+        return meta_node_to_json_schema(self, options)
+
+    def reset_inheritance_caches(self) -> None:
+        """Reset all inheritance-related caches for this node."""
+
+        self._inherited_meta_nodes = None
+        self._prop_types = None
+        self._attributes = None
+        self._inherited_properties = None
+
+
+# ---------------------------------------------------------------------------
+# Sample MetaNode definitions
+# ---------------------------------------------------------------------------
+
+
+def create_sample_meta_nodes(store: "ElementStore") -> None:
+    """Create a few sample MetaNodes with aliases and options.
+
+    These samples are intended for tests, demos and local experimentation.
+    They mirror the TypeScript meta examples and ensure that the
+    NaturalQueryParser can discover:
+      - property aliases (e.g. "disorder", "disorder type")
+      - option-type properties with explicit option labels
+    """
+
+    # Person meta node (simple example without options)
+    person_props = MetaNodeProperties(
+        name="Person",
+        type="node",
+        property_types={
+            "name": MetaPropertyType(
+                type=PropValueType.STRING,
+                label="Name",
+                key="name",
+                aliases=["full name"],
+            ),
+            "age": MetaPropertyType(
+                type=PropValueType.NUMBER,
+                label="Age",
+                key="age",
+                aliases=["years"],
+            ),
+        },
+    )
+    person_details = MetaNodeDetails(
+        id="meta_person",
+        class_id="person",
+        type=NodeTypes.META,
+        properties=person_props,
+    )
+    person_meta = MetaNode(person_details, store)
+    store.elements[person_meta.id] = person_meta
+
+    # Event meta node with an OPTIONS property and aliases
+    event_props = MetaNodeProperties(
+        name="Event",
+        type="node",
+        property_types={
+            "disorder_type": MetaPropertyType(
+                type=PropValueType.OPTIONS,
+                label="Disorder type",
+                key="disorder_type",
+                options=[
+                    "Political violence",
+                    "Demonstrations",
+                    "Strategic developments",
+                ],
+                # These aliases should be usable in natural queries like
+                # "events with disorder Political violence" or
+                # "events with disorder type Political violence".
+                aliases=["disorder", "disorder type"],
+            )
+        },
+    )
+    event_details = MetaNodeDetails(
+        id="meta_event",
+        class_id="event",
+        type=NodeTypes.META,
+        properties=event_props,
+    )
+    event_meta = MetaNode(event_details, store)
+    store.elements[event_meta.id] = event_meta
+
     @property
     def filterable_properties(self) -> List[MetaPropertyType]:
         """Get properties that can be used for filtering."""
@@ -521,6 +833,9 @@ class MetaNode(BaseElement):
     @lru_cache(maxsize=64)
     def _calculate_inherited_meta_nodes(self) -> List["MetaNode"]:
         """Calculate inherited meta nodes recursively."""
+        if not self.store:
+            return []
+
         visited_nodes = set()
         inherited_meta_nodes = []
 
@@ -718,6 +1033,9 @@ class MetaNode(BaseElement):
         self, pt: Union[str, MetaPropertyType], force: bool = False
     ) -> None:
         """Calculate unique values for a property type."""
+        if not self.store:
+            return
+
         if isinstance(pt, str):
             if pt not in self.all_prop_types:
                 return
@@ -748,3 +1066,53 @@ class MetaNode(BaseElement):
                             values[val] = values.get(val, 0) + 1
 
         prop_type._unique = values
+
+    def to_dict_api(self) -> dict:
+        """Convert metanode to dictionary for API response with classId format and name in properties."""
+        from typing import Any
+
+        def convert_to_serializable(obj: Any) -> Any:
+            """Convert generators and other non-serializable objects to serializable forms."""
+            if isinstance(obj, dict):
+                return {k: convert_to_serializable(v) for k, v in obj.items()}
+            elif isinstance(obj, (list, tuple)):
+                return [convert_to_serializable(item) for item in obj]
+            elif hasattr(obj, "__iter__") and not isinstance(obj, (str, bytes)):
+                # Convert generators and other iterables to lists
+                try:
+                    return [convert_to_serializable(item) for item in obj]
+                except TypeError:
+                    return obj
+            return obj
+
+        # Build the properties dict
+        properties_dict = {}
+        if self.properties:
+            if isinstance(self.properties, dict):
+                properties_dict = convert_to_serializable(self.properties)
+            else:
+                # Convert properties object to dict
+                props_dict = (
+                    vars(self.properties)
+                    if hasattr(self.properties, "__dict__")
+                    else dict(self.properties)
+                )
+                properties_dict = convert_to_serializable(props_dict)
+
+        # Build attributes dict
+        attributes_dict = {}
+        if self.attributes:
+            attributes_dict = convert_to_serializable(self.attributes)
+
+        result = {
+            "id": self.id,
+            "type": self.type.value if hasattr(self.type, "value") else str(self.type),
+            "classId": self.class_id,
+            "properties": properties_dict,
+            "source": self.source,
+        }
+
+        if attributes_dict:
+            result["attributes"] = attributes_dict
+
+        return result
